@@ -45,6 +45,7 @@
 
 #include <call-ui.h>
 #include <glib/gi18n.h>
+#include <glib-unix.h>
 #include <handy.h>
 #include <libcallaudio.h>
 
@@ -57,9 +58,8 @@
 
 #define DEFAULT_PROVIDER_PLUGIN "mm"
 
-struct _CallsApplication
-{
-  GtkApplication parent_instance;
+struct _CallsApplication {
+  GtkApplication    parent_instance;
 
   gboolean          daemon;
   CallsRinger      *ringer;
@@ -70,19 +70,30 @@ struct _CallsApplication
   CallsDBusManager *dbus_manager;
 
   char             *uri;
+  guint             id_sigterm;
+  guint             id_sigint;
 };
 
 G_DEFINE_TYPE (CallsApplication, calls_application, GTK_TYPE_APPLICATION);
 
 
-static gboolean start_proper (CallsApplication *self);
-
+static void start_proper (CallsApplication *self);
 
 static gboolean
-cmd_verbose_cb (const char  *option_name,
-                const char  *value,
-                gpointer     data,
-                GError     **error)
+on_int_or_term_signal (CallsApplication *self)
+{
+  g_debug ("Received SIGTERM/SIGINT, shutting down gracefully");
+
+  g_application_quit (G_APPLICATION (self));
+
+  return G_SOURCE_REMOVE;
+}
+
+static gboolean
+cmd_verbose_cb (const char *option_name,
+                const char *value,
+                gpointer    data,
+                GError    **error)
 {
   calls_log_increase_verbosity ();
 
@@ -165,6 +176,7 @@ set_default_providers_action (GSimpleAction *action,
 {
   CallsManager *manager = calls_manager_get_default ();
   CallsSettings *settings = calls_manager_get_settings (manager);
+
   g_auto (GStrv) plugins = NULL;
   /**
    * Only add default providers when there are none added yet,
@@ -186,16 +198,12 @@ set_daemon_action (GSimpleAction *action,
                    gpointer       user_data)
 {
   CallsApplication *self = CALLS_APPLICATION (user_data);
+  gboolean daemon = g_variant_get_boolean (parameter);
 
-  if (self->main_window) {
-    g_warning ("Cannot set application as a daemon"
-               " because application is already started");
-    return;
-  }
+  self->daemon = daemon;
 
-  self->daemon = TRUE;
-
-  g_debug ("Application marked as daemon");
+  g_debug ("Application %smarked as daemon",
+           daemon ? "" : "not ");
 }
 
 
@@ -240,7 +248,7 @@ extract_dial_string (const char *number)
   }
 
   dial_string = g_regex_replace_literal
-    (replace_visual, number, -1, 0, "", 0, &error);
+                  (replace_visual, number, -1, 0, "", 0, &error);
 
   if (!dial_string) {
     g_warning ("Error replacing visual separators"
@@ -260,25 +268,25 @@ call_number (CallsApplication *self,
   g_autofree char *dial_string = NULL;
   gboolean number_ok;
 
- number_ok = check_dial_number (number);
- if (!number_ok) {
-   g_warning ("Dial number `%s' is not a valid dial string",
-              number);
-   return;
- }
+  number_ok = check_dial_number (number);
+  if (!number_ok) {
+    g_warning ("Dial number `%s' is not a valid dial string",
+               number);
+    return;
+  }
 
- dial_string = extract_dial_string (number);
- if (!dial_string) {
-   return;
- }
+  dial_string = extract_dial_string (number);
+  if (!dial_string) {
+    return;
+  }
 
- g_debug ("Dialing dial string `%s' extracted from number `%s'",
-          dial_string, number);
+  g_debug ("Dialing dial string `%s' extracted from number `%s'",
+           dial_string, number);
 
- start_proper (self);
+  start_proper (self);
 
- calls_main_window_dial (self->main_window,
-                         dial_string);
+  calls_main_window_dial (self->main_window,
+                          dial_string);
 
 }
 
@@ -309,16 +317,17 @@ open_tel_uri (CallsApplication *self,
               const char       *uri)
 {
   const char *number = NULL;
+  g_autofree char* uri_str = g_uri_unescape_string (uri, NULL);
 
   g_debug ("Opening tel URI `%s'", uri);
 
-  number = &uri[4]; // tel:NUMBER
+  number = &uri_str[4]; // tel:NUMBER
   if (!number || !*number) {
     g_autofree char *msg =
       g_strdup_printf (_("Tried dialing invalid tel URI `%s'"), uri);
 
     calls_message_source_emit_message (CALLS_MESSAGE_SOURCE (calls_manager_get_default ()),
-                                       "msg",
+                                       msg,
                                        GTK_MESSAGE_WARNING);
     g_warning ("Ignoring invalid tel URI `%s'", uri);
     return;
@@ -348,6 +357,7 @@ dial_action (GSimpleAction *action,
     call_number (self, number);
 }
 
+
 static void
 copy_number (GSimpleAction *action,
              GVariant      *parameter,
@@ -362,12 +372,14 @@ copy_number (GSimpleAction *action,
   g_debug ("Copied `%s' to clipboard", number);
 }
 
+
 static void
 show_accounts (GSimpleAction *action,
                GVariant      *parameter,
                gpointer       user_data)
 {
-  CallsApplication *app = CALLS_APPLICATION (g_application_get_default ());
+  CallsApplication *app = CALLS_APPLICATION (user_data);
+
   calls_main_window_show_accounts_overview (app->main_window);
 }
 
@@ -377,7 +389,7 @@ manager_state_changed_cb (GApplication *application)
   GAction* dial_action = g_action_map_lookup_action (G_ACTION_MAP (application), "dial");
   CallsManagerFlags state_flags = calls_manager_get_state_flags (calls_manager_get_default ());
   gboolean enabled = (state_flags & CALLS_MANAGER_FLAGS_HAS_CELLULAR_MODEM) ||
-    (state_flags & CALLS_MANAGER_FLAGS_HAS_VOIP_ACCOUNT);
+                     (state_flags & CALLS_MANAGER_FLAGS_HAS_VOIP_ACCOUNT);
 
   g_simple_action_set_enabled (G_SIMPLE_ACTION (dial_action), enabled);
 }
@@ -386,11 +398,11 @@ static const GActionEntry actions[] =
 {
   { "set-provider-names", set_provider_names_action, "as" },
   { "set-default-providers", set_default_providers_action, NULL },
-  { "set-daemon", set_daemon_action, NULL },
+  { "set-daemon", set_daemon_action, "b" },
   { "dial", dial_action, "s" },
-  { "copy-number", copy_number, "s"},
+  { "copy-number", copy_number, "s" },
   /* TODO About dialog { "about", show_about, NULL}, */
-  { "accounts", show_accounts, NULL},
+  { "accounts", show_accounts, NULL },
 };
 
 
@@ -398,12 +410,21 @@ static int
 calls_application_handle_local_options (GApplication *application,
                                         GVariantDict *options)
 {
+  guint verbosity = calls_log_get_verbosity ();
+
   if (g_variant_dict_contains (options, "version")) {
     g_print ("%s %s\n", APP_DATA_NAME, *VCS_TAG ? VCS_TAG : PACKAGE_VERSION);
 
     return 0;
   }
 
+  /* Propagate verbosity changes to the main instance */
+  if (verbosity > 0) {
+    g_variant_dict_insert_value (options, "verbosity", g_variant_new_uint32 (verbosity));
+    g_print ("Increasing verbosity to %u\n", verbosity);
+  }
+
+
   return -1;
 }
 
@@ -413,7 +434,7 @@ startup (GApplication *application)
 {
   g_autoptr (GtkCssProvider) provider = NULL;
   g_autoptr (GError) error = NULL;
-#if HDY_CHECK_VERSION(1, 5, 0)
+#if HDY_CHECK_VERSION (1, 5, 0)
   HdyStyleManager *style_manager;
 #endif
 
@@ -421,7 +442,7 @@ startup (GApplication *application)
 
   hdy_init ();
 
-#if HDY_CHECK_VERSION(1, 5, 0)
+#if HDY_CHECK_VERSION (1, 5, 0)
   style_manager = hdy_style_manager_get_default ();
 
   hdy_style_manager_set_color_scheme (style_manager, HDY_COLOR_SCHEME_PREFER_LIGHT);
@@ -462,9 +483,11 @@ calls_application_command_line (GApplication            *application,
   CallsApplication *self = CALLS_APPLICATION (application);
   GVariantDict *options;
   const char *arg;
+
   g_autoptr (GVariant) providers = NULL;
-  g_auto(GStrv) arguments = NULL;
+  g_auto (GStrv) arguments = NULL;
   gint argc;
+  guint verbosity;
 
   options = g_application_command_line_get_options_dict (command_line);
 
@@ -479,14 +502,25 @@ calls_application_command_line (GApplication            *application,
                                     NULL);
   }
 
-  if (g_variant_dict_contains (options, "daemon"))
-    g_action_group_activate_action (G_ACTION_GROUP (application),
-                                    "set-daemon", NULL);
+  g_action_group_activate_action (G_ACTION_GROUP (application),
+                                  "set-daemon",
+                                  g_variant_new_boolean (g_variant_dict_contains (options, "daemon")));
 
   if (g_variant_dict_lookup (options, "dial", "&s", &arg))
     g_action_group_activate_action (G_ACTION_GROUP (application),
                                     "dial", g_variant_new_string (arg));
 
+  /* TODO make this a comma separated string of "CATEGORY:level" pairs */
+  if (g_variant_dict_lookup (options, "verbosity", "u", &verbosity)) {
+    gint delta = calls_log_set_verbosity (verbosity);
+    guint level = calls_log_get_verbosity ();
+    if (delta != 0)
+      g_print ("%s verbosity by %d to %u\n",
+               delta > 0 ? "Increased" : "Decreased",
+               delta < 0 ? -1 * delta : delta,
+               level);
+  }
+
   arguments = g_application_command_line_get_arguments (command_line, &argc);
 
   /* Keep only the first URI, if there are many */
@@ -514,8 +548,8 @@ app_shutdown (GApplication *application)
 
 
 static void
-notify_window_visible_cb (GtkWidget       *window,
-                          GParamSpec      *pspec,
+notify_window_visible_cb (GtkWidget        *window,
+                          GParamSpec       *pspec,
                           CallsApplication *application)
 {
   CallsManager *manager = calls_manager_get_default ();
@@ -529,13 +563,13 @@ notify_window_visible_cb (GtkWidget       *window,
 }
 
 
-static gboolean
-start_proper (CallsApplication  *self)
+static void
+start_proper (CallsApplication *self)
 {
   GtkApplication *gtk_app;
 
   if (self->main_window) {
-    return TRUE;
+    return;
   }
 
   gtk_app = GTK_APPLICATION (self);
@@ -550,8 +584,8 @@ start_proper (CallsApplication  *self)
   g_assert (CALLS_IS_NOTIFIER (self->notifier));
 
   self->main_window = calls_main_window_new
-    (gtk_app,
-     G_LIST_MODEL (self->record_store));
+                        (gtk_app,
+                        G_LIST_MODEL (self->record_store));
   g_assert (self->main_window != NULL);
 
   self->call_window = calls_call_window_new (gtk_app);
@@ -561,10 +595,9 @@ start_proper (CallsApplication  *self)
                     "notify::visible",
                     G_CALLBACK (notify_window_visible_cb),
                     self);
-
-  return TRUE;
 }
 
+
 static void
 activate (GApplication *application)
 {
@@ -573,15 +606,8 @@ activate (GApplication *application)
 
   g_debug ("Activated");
 
-  if (self->main_window) {
-    present = TRUE;
-  } else {
-    gboolean ok = start_proper (self);
-    if (!ok)
-      return;
-
-    present = !self->daemon;
-  }
+  start_proper (self);
+  present = !self->daemon;
 
   if (present || self->uri) {
     gtk_window_present (GTK_WINDOW (self->main_window));
@@ -599,11 +625,12 @@ activate (GApplication *application)
   g_clear_pointer (&self->uri, g_free);
 }
 
+
 static void
-app_open (GApplication  *application,
-          GFile        **files,
-          gint           n_files,
-          const char    *hint)
+app_open (GApplication *application,
+          GFile       **files,
+          gint          n_files,
+          const char   *hint)
 {
   CallsApplication *self = CALLS_APPLICATION (application);
 
@@ -642,7 +669,10 @@ app_open (GApplication  *application,
 static void
 finalize (GObject *object)
 {
-  CallsApplication *self = (CallsApplication *)object;
+  CallsApplication *self = (CallsApplication *) object;
+
+  g_clear_handle_id (&self->id_sigterm, g_source_remove);
+  g_clear_handle_id (&self->id_sigint, g_source_remove);
 
   g_clear_object (&self->call_window);
   g_clear_object (&self->main_window);
@@ -669,8 +699,8 @@ calls_application_class_init (CallsApplicationClass *klass)
   application_class->shutdown = app_shutdown;
   application_class->activate = activate;
   application_class->open = app_open;
-  application_class->dbus_register  = calls_application_dbus_register;
-  application_class->dbus_unregister  = calls_application_dbus_unregister;
+  application_class->dbus_register = calls_application_dbus_register;
+  application_class->dbus_unregister = calls_application_dbus_unregister;
 
   g_type_ensure (CALLS_TYPE_HISTORY_BOX);
   g_type_ensure (CALLS_TYPE_NEW_CALL_BOX);
@@ -716,6 +746,12 @@ calls_application_init (CallsApplication *self)
     }
   };
 
+  self->id_sigterm = g_unix_signal_add (SIGTERM, G_SOURCE_FUNC (on_int_or_term_signal), self);
+  g_source_set_name_by_id (self->id_sigterm, "signal: SIGTERM handler");
+
+  self->id_sigint = g_unix_signal_add (SIGINT, G_SOURCE_FUNC (on_int_or_term_signal), self);
+  g_source_set_name_by_id (self->id_sigint, "signal: SIGINT handler");
+
   g_application_add_main_option_entries (G_APPLICATION (self), options);
 }
 
