@@ -24,11 +24,12 @@
 
 #define G_LOG_DOMAIN "CallsRecordStore"
 
-#include "calls-record-store.h"
+#include "config.h"
+
 #include "calls-call-record.h"
 #include "calls-manager.h"
-#include "calls-call.h"
-#include "config.h"
+#include "calls-ui-call-data.h"
+#include "calls-record-store.h"
 
 #include <gom/gom.h>
 #include <glib/gi18n.h>
@@ -50,20 +51,18 @@ typedef enum
 
 
 static CallsCallRecordState
-state_to_record_state (CallsCallState call_state)
+state_to_record_state (CuiCallState call_state)
 {
   switch (call_state) {
-  case CALLS_CALL_STATE_DIALING:
-  case CALLS_CALL_STATE_ALERTING:
-  case CALLS_CALL_STATE_INCOMING:
-  case CALLS_CALL_STATE_WAITING:
+  case CUI_CALL_STATE_CALLING:
+  case CUI_CALL_STATE_INCOMING:
     return STARTED;
 
-  case CALLS_CALL_STATE_ACTIVE:
-  case CALLS_CALL_STATE_HELD:
+  case CUI_CALL_STATE_ACTIVE:
+  case CUI_CALL_STATE_HELD:
     return ANSWERED;
 
-  case CALLS_CALL_STATE_DISCONNECTED:
+  case CUI_CALL_STATE_DISCONNECTED:
     return ENDED;
 
   default:
@@ -72,13 +71,12 @@ state_to_record_state (CallsCallState call_state)
 }
 
 
-struct _CallsRecordStore
-{
+struct _CallsRecordStore {
   GtkApplicationWindow parent_instance;
 
-  gchar *filename;
-  GomAdapter *adapter;
-  GomRepository *repository;
+  gchar               *filename;
+  GomAdapter          *adapter;
+  GomRepository       *repository;
 };
 
 G_DEFINE_TYPE (CallsRecordStore, calls_record_store, G_TYPE_LIST_STORE);
@@ -121,8 +119,8 @@ delete_record_cb (GomResource      *resource,
 
 
 static void
-delete_call_cb (CallsCallRecord    *record,
-                CallsRecordStore   *self)
+delete_call_cb (CallsCallRecord  *record,
+                CallsRecordStore *self)
 {
   gom_resource_delete_async (GOM_RESOURCE (record),
                              (GAsyncReadyCallback) delete_record_cb,
@@ -135,8 +133,8 @@ load_calls_fetch_cb (GomResourceGroup *group,
                      GAsyncResult     *res,
                      CallsRecordStore *self)
 {
-  gboolean ok;
   g_autoptr (GError) error = NULL;
+  gboolean ok;
   guint count, i;
   gpointer *records;
 
@@ -158,7 +156,6 @@ load_calls_fetch_cb (GomResourceGroup *group,
   for (i = 0; i < count; ++i) {
     GomResource *resource;
     CallsCallRecord *record;
-    GDateTime *end = NULL;
 
     resource = gom_resource_group_get_index (group, i);
     g_assert (resource != NULL);
@@ -167,12 +164,6 @@ load_calls_fetch_cb (GomResourceGroup *group,
 
     records[i] = record;
 
-    g_object_get (G_OBJECT (record),
-                  "end", &end,
-                  NULL);
-    if (end)
-      g_date_time_unref (end);
-
     g_signal_connect (record,
                       "call-delete",
                       G_CALLBACK (delete_call_cb),
@@ -195,8 +186,8 @@ load_calls_find_cb (GomRepository    *repository,
                     GAsyncResult     *res,
                     CallsRecordStore *self)
 {
-  GomResourceGroup *group;
   g_autoptr (GError) error = NULL;
+  GomResourceGroup *group;
   guint count;
 
   group = gom_repository_find_finish (repository,
@@ -218,12 +209,11 @@ load_calls_find_cb (GomRepository    *repository,
 
   g_debug ("Found %u call records in database `%s', fetching",
            count, self->filename);
-  gom_resource_group_fetch_async
-    (group,
-     0,
-     count,
-     (GAsyncReadyCallback)load_calls_fetch_cb,
-     self);
+  gom_resource_group_fetch_async (group,
+                                  0,
+                                  count,
+                                  (GAsyncReadyCallback) load_calls_fetch_cb,
+                                  self);
 }
 
 
@@ -234,7 +224,7 @@ load_calls (CallsRecordStore *self)
   GomSorting *sorting;
 
   filter = gom_filter_new_is_not_null
-    (CALLS_TYPE_CALL_RECORD, "start");
+             (CALLS_TYPE_CALL_RECORD, "start");
 
   sorting = gom_sorting_new (CALLS_TYPE_CALL_RECORD,
                              "start",
@@ -247,7 +237,7 @@ load_calls (CallsRecordStore *self)
                                     CALLS_TYPE_CALL_RECORD,
                                     filter,
                                     sorting,
-                                    (GAsyncReadyCallback)load_calls_find_cb,
+                                    (GAsyncReadyCallback) load_calls_find_cb,
                                     self);
 
   g_object_unref (G_OBJECT (filter));
@@ -255,8 +245,8 @@ load_calls (CallsRecordStore *self)
 
 
 static void
-set_up_repo_migrate_cb (GomRepository *repo,
-                        GAsyncResult *res,
+set_up_repo_migrate_cb (GomRepository    *repo,
+                        GAsyncResult     *res,
                         CallsRecordStore *self)
 {
   g_autoptr (GError) error = NULL;
@@ -264,17 +254,17 @@ set_up_repo_migrate_cb (GomRepository *repo,
 
   ok = gom_repository_automatic_migrate_finish (repo, res, &error);
   if (!ok) {
-      if (error)
-        g_warning ("Error migrating call record database `%s': %s",
-                   self->filename, error->message);
-      else
-        g_warning ("Unknown error migrating call record database `%s'",
-                   self->filename);
+    if (error)
+      g_warning ("Error migrating call record database `%s': %s",
+                 self->filename, error->message);
+    else
+      g_warning ("Unknown error migrating call record database `%s'",
+                 self->filename);
 
-      g_clear_object (&self->repository);
-      g_clear_object (&self->adapter);
+    g_clear_object (&self->repository);
+    g_clear_object (&self->adapter);
 
-    } else {
+  } else {
     g_debug ("Successfully migrated call record database `%s'",
              self->filename);
     load_calls (self);
@@ -300,13 +290,13 @@ set_up_repo (CallsRecordStore *self)
   g_debug ("Attempting migration of call"
            " record database `%s'",
            self->filename);
-  types = g_list_append (types, (gpointer)CALLS_TYPE_CALL_RECORD);
+  types = g_list_append (types, (gpointer) CALLS_TYPE_CALL_RECORD);
   gom_repository_automatic_migrate_async
     (repo,
-     RECORD_STORE_VERSION,
-     types,
-     (GAsyncReadyCallback)set_up_repo_migrate_cb,
-     self);
+    RECORD_STORE_VERSION,
+    types,
+    (GAsyncReadyCallback) set_up_repo_migrate_cb,
+    self);
 
   self->repository = repo;
 }
@@ -318,28 +308,27 @@ close_adapter (CallsRecordStore *self)
   g_autoptr (GError) error = NULL;
   gboolean ok;
 
-  if (!self->adapter)
-    {
-      return;
-    }
+  if (!self->adapter) {
+    return;
+  }
 
-  ok = gom_adapter_close_sync(self->adapter, &error);
+  ok = gom_adapter_close_sync (self->adapter, &error);
   if (!ok) {
-      if (error)
-        g_warning ("Error closing call record database `%s': %s",
-                   self->filename, error->message);
-      else
-        g_warning ("Unknown error closing call record database `%s'",
-                   self->filename);
-    }
+    if (error)
+      g_warning ("Error closing call record database `%s': %s",
+                 self->filename, error->message);
+    else
+      g_warning ("Unknown error closing call record database `%s'",
+                 self->filename);
+  }
 
   g_clear_object (&self->adapter);
 }
 
 
 static void
-open_repo_adapter_open_cb (GomAdapter *adapter,
-                           GAsyncResult *res,
+open_repo_adapter_open_cb (GomAdapter       *adapter,
+                           GAsyncResult     *res,
                            CallsRecordStore *self)
 {
   g_autoptr (GError) error = NULL;
@@ -388,18 +377,17 @@ open_repo (CallsRecordStore *self)
   self->adapter = gom_adapter_new ();
   gom_adapter_open_async
     (self->adapter,
-     uri,
-     (GAsyncReadyCallback)open_repo_adapter_open_cb,
-     self);
+    uri,
+    (GAsyncReadyCallback) open_repo_adapter_open_cb,
+    self);
 
   g_free (uri);
 }
 
 
-struct CallsRecordCallData
-{
+struct CallsRecordCallData {
   CallsRecordStore *self;
-  CallsCall *call;
+  CallsUiCallData  *call;
 };
 
 
@@ -409,6 +397,7 @@ record_call_save_cb (GomResource                *resource,
                      struct CallsRecordCallData *data)
 {
   GObject * const call_obj = G_OBJECT (data->call);
+
   g_autoptr (GError) error = NULL;
   gboolean ok;
 
@@ -439,23 +428,27 @@ record_call_save_cb (GomResource                *resource,
 
 static void
 record_call (CallsRecordStore *self,
-             CallsCall        *call)
+             CallsUiCallData  *call)
 {
   GObject * const call_obj = G_OBJECT (call);
   GDateTime *start;
   CallsCallRecord *record;
   struct CallsRecordCallData *data;
+  gboolean inbound;
+  g_autofree char *protocol = NULL;
 
   g_assert (g_object_get_data (call_obj, "calls-call-record") == NULL);
 
   start = g_object_get_data (call_obj, "calls-call-start");
   g_assert (start != NULL);
 
+  g_object_get (call, "inbound", &inbound, "protocol", &protocol, NULL);
+
   record = g_object_new (CALLS_TYPE_CALL_RECORD,
                          "repository", self->repository,
-                         "target", calls_call_get_number (call),
-                         "inbound", calls_call_get_inbound (call),
-                         "protocol", calls_call_get_protocol (call),
+                         "target", cui_call_get_id (CUI_CALL (call)),
+                         "inbound", inbound,
+                         "protocol", protocol,
                          "start", start,
                          NULL);
 
@@ -469,7 +462,7 @@ record_call (CallsRecordStore *self,
   data->call = call;
 
   gom_resource_save_async (GOM_RESOURCE (record),
-                           (GAsyncReadyCallback)record_call_save_cb,
+                           (GAsyncReadyCallback) record_call_save_cb,
                            data);
 }
 
@@ -497,8 +490,8 @@ update_cb (GomResource  *resource,
 
 
 static void
-stamp_call (CallsCallRecord  *record,
-            const gchar      *stamp_name)
+stamp_call (CallsCallRecord *record,
+            const gchar     *stamp_name)
 {
   GObject *record_obj = G_OBJECT (record);
   GDateTime *stamp = NULL;
@@ -519,22 +512,24 @@ stamp_call (CallsCallRecord  *record,
   g_date_time_unref (stamp);
 
   gom_resource_save_async (GOM_RESOURCE (record),
-                           (GAsyncReadyCallback)update_cb,
+                           (GAsyncReadyCallback) update_cb,
                            NULL);
 }
 
 
 static void
 state_changed_cb (CallsRecordStore *self,
-                  CallsCallState    new_state,
-                  CallsCallState    old_state,
-                  CallsCall        *call)
+                  CuiCallState      new_state,
+                  CuiCallState      old_state,
+                  CallsUiCallData  *call)
 {
   GObject *call_obj = G_OBJECT (call);
   CallsCallRecord *record =
     g_object_get_data (call_obj, "calls-call-record");
   CallsCallRecordState new_rec_state, old_rec_state;
 
+  g_debug ("Call state changed from %d to %d",
+           old_state, new_state);
 
   /* Check whether the call is recorded */
   if (!record) {
@@ -595,7 +590,7 @@ state_changed_cb (CallsRecordStore *self,
 
 static void
 call_added_cb (CallsRecordStore *self,
-               CallsCall        *call)
+               CallsUiCallData  *call)
 {
   GObject * const call_obj = G_OBJECT (call);
   GDateTime *start;
@@ -603,7 +598,7 @@ call_added_cb (CallsRecordStore *self,
   g_assert (g_object_get_data (call_obj, "calls-call-start") == NULL);
   start = g_date_time_new_now_utc ();
   g_object_set_data_full (call_obj, "calls-call-start",
-                          start, (GDestroyNotify)g_date_time_unref);
+                          start, (GDestroyNotify) g_date_time_unref);
 
   if (!self->repository) {
     open_repo (self);
@@ -621,7 +616,7 @@ call_added_cb (CallsRecordStore *self,
 
 static void
 call_removed_cb (CallsRecordStore *self,
-                 CallsCall        *call,
+                 CallsUiCallData  *call,
                  const gchar      *reason)
 {
   /* Stamp the call as ended if it hasn't already been done */
@@ -645,12 +640,12 @@ constructed (GObject *object)
   open_repo (self);
 
   g_signal_connect_swapped (calls_manager_get_default (),
-                            "call-add",
+                            "ui-call-added",
                             G_CALLBACK (call_added_cb),
                             self);
 
   g_signal_connect_swapped (calls_manager_get_default (),
-                            "call-remove",
+                            "ui-call-removed",
                             G_CALLBACK (call_removed_cb),
                             self);
 
@@ -671,7 +666,6 @@ dispose (GObject *object)
   g_list_store_remove_all (G_LIST_STORE (self));
 
   g_clear_object (&self->repository);
-  close_adapter (self);
 
   G_OBJECT_CLASS (calls_record_store_parent_class)->dispose (object);
 }
@@ -683,6 +677,7 @@ finalize (GObject *object)
   CallsRecordStore *self = CALLS_RECORD_STORE (object);
 
   g_free (self->filename);
+  close_adapter (self);
 
   G_OBJECT_CLASS (calls_record_store_parent_class)->finalize (object);
 }
